/*
 * "Hello World" example.
 *
 * This example prints 'Hello from Nios II' to the STDOUT stream. It runs on
 * the Nios II 'standard', 'full_featured', 'fast', and 'low_cost' example
 * designs. It runs with or without the MicroC/OS-II RTOS and requires a STDOUT
 * device in your system's hardware.
 * The memory footprint of this hosted application is ~69 kbytes by default
 * using the standard reference design.
 *
 * For a reduced footprint version of this template, and an explanation of how
 * to reduce the memory footprint for a given application, see the
 * "small_hello_world" template.
 *
 */

#include <stdio.h>
#include <unistd.h>
#include <io.h>
#include <time.h>
#include "system.h"

//#define DPRINTF(X)  printf((x))

void auto_select( void );
int flash_chip_erase( void );
int flash_sector_erase( unsigned int sector );
int flash_write( unsigned int addr, unsigned short data );
unsigned short flash_read( unsigned int addr );
void check_7seg_leds( int seg_led_number,            // 0 7seg #0, 1 7seg #1
		              unsigned char disp_num );      // 0 to F
int check_flash( void );
int sram_check( void );
int ddr_check( void );
void check_7_led( void );



const int FLASH_TIMEOUT = 1000000;
const unsigned int DDR_SIZE   = (MEM_IF_DDR3_EMIF_0_SPAN);
const unsigned int SRAM_SIZE  = (SRAM_CONTOLLER_SPAN);
const unsigned int FLASH_SIZE = (FLASH_CONTOLLER_SPAN);

#define COLUM_ADDR_BITS      (10)
#define DDR_BUS_WIDTH        (2)
#define DDR_COLUM_ADDR_SPACE (1 << (COLUM_ADDR_BITS+DDR_BUS_WIDTH))

#define FLASH_SECTOR_SIZE (128*1024)
#define TEST_OFFSET (2)
#define NUMBER_OF_SECTORS (256)



volatile unsigned char *p_led_select0 = (unsigned char *)LED_SELECT0_BASE;
volatile unsigned char *p_led_select1 = (unsigned char *)LED_SELECT1_BASE;
volatile unsigned char *p_led_select2 = (unsigned char *)LED_SELECT2_BASE;

//DDR check
int ddr_check(void)
{
// DDR size@256MBytes(128MByte x 2)   bank address:3bits  row address:13bits   column address:10bits

	unsigned int value, *p;
	unsigned char *p1;
	unsigned short *p2;

#if 0
	for( value = 0, p = (unsigned short *)MEM_IF_DDR3_EMIF_0_BASE; p < (unsigned short *)(MEM_IF_DDR3_EMIF_0_BASE+DDR_SIZE); value++, p++ ) {
		*p = value;
	}
	for( value = 0, p = (unsigned short *)MEM_IF_DDR3_EMIF_0_BASE; p < (unsigned short *)(MEM_IF_DDR3_EMIF_0_BASE+DDR_SIZE); value++, p++ ) {
		if( *p != value ) {
			return -1;			//Fail
		}
	}
#endif



	//Check changing all column address(10 bits and 4 bytes access so clolum adress space 2^(10+2) )
	for( value = 0, p = (unsigned int *)MEM_IF_DDR3_EMIF_0_BASE; p < (unsigned int *)(MEM_IF_DDR3_EMIF_0_BASE+DDR_COLUM_ADDR_SPACE); value++, p++ ) {
		*p = value;
	}
	for( value = 0, p = (unsigned int *)MEM_IF_DDR3_EMIF_0_BASE; p < (unsigned int *)(MEM_IF_DDR3_EMIF_0_BASE+DDR_COLUM_ADDR_SPACE); value++, p++ ) {
		if( *p != value ) {
			return -1;			//Fail
		}
	}

	//Check changing bank address and row address( column address 10nit and each 4byte access, so we have to increment address DDR_COLUM_ADDR_SPACE
	for( value = 0, p = (unsigned int *)MEM_IF_DDR3_EMIF_0_BASE; p < (unsigned int *)(MEM_IF_DDR3_EMIF_0_BASE+DDR_SIZE); value++, p+=(DDR_COLUM_ADDR_SPACE>>2) ) {
		*p = value;
	}
	for( value = 0, p = (unsigned int *)MEM_IF_DDR3_EMIF_0_BASE; p < (unsigned int *)(MEM_IF_DDR3_EMIF_0_BASE+DDR_SIZE); value++, p+=(DDR_COLUM_ADDR_SPACE>>2) ) {
		if( *p != value ) {
			return -1;			//Fail
		}
	}


	//Check Byte Enable signals accessing 1byte data.
	p1 = (unsigned char*)MEM_IF_DDR3_EMIF_0_BASE;
	*p1     = 0x0;
	*(p1+1) = 0x1;
	*(p1+2) = 0x2;
	*(p1+3) = 0x3;
	if( *p1 != 0  || *(p1+1) != 0x1 || *(p1+2) != 0x2 || *(p1+3) != 0x3 ) {
		return -1;
	}
	//Check Byte Enable signals accessing 2byte data.
	p2 = (unsigned short *)MEM_IF_DDR3_EMIF_0_BASE;
	*p2     = 0x0;
	*(p2+1) = 0x1;
	if( *p2 != 0 || *(p2+1) != 0x1 ) {
		return -1;
	}
	return 0;					//Pass
}

//SRAM check
int sram_check( void )
{
	unsigned short value, *p;
	unsigned char  *p1;


	for( value = 0, p = (unsigned short *)SRAM_CONTOLLER_BASE; p < (unsigned short *)(SRAM_CONTOLLER_BASE+SRAM_SIZE); value++, p++ ) {
		*p = value;
	}
	for( value = 0, p = (unsigned short *)SRAM_CONTOLLER_BASE; p < (unsigned short *)(SRAM_CONTOLLER_BASE+SRAM_SIZE); value++, p++ ) {
		if( *p != value ) {
			return -1; //Fail
		}
	}
	//Check Byte Enable signals accessing 1byte data.
	p1 = (unsigned char *)SRAM_CONTOLLER_BASE;
	*p1     = 0x0;
	*(p1+1) = 0x1;
	if( *p1 != 0  || *(p1+1) != 0x1  ) {
		return -1;
	}
	return 0;	//Pass
}


//Functions for Flash memory Access
void auto_select(void)
{
	unsigned short manuf_id, device_id_1, device_id_2, device_id_3, secure_device, sector_protect;

	/* Auto Select Entry */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x00AA; /* write unlock cycle 1 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x2AA ) = 0x0055; /* write unlock cycle 2 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x0090; /* write autoselect command */
	/* multiple reads can be performed after entry */
	manuf_id = *( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x000 ); /* read manuf. id */
    device_id_1 = *( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x001 ); /* read device_id1 */
    device_id_2 = *( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x00E ); /* read device_id2*/
    device_id_3 = *( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x00F ); /* read manuf. id3 */
	secure_device  = *( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x003 ); /* read device_id1 */
	sector_protect = *( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x002 ); /* read device_id1 */
	/* Autoselect exit */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x000 ) = 0x00F0; /* exit autoselect (write reset command) */
}

//Flash erase
int flash_chip_erase(void)
{
	unsigned short rdata1, rdata2;
	int timeout = 0;

	/* Note: Cannot be suspended */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x00AA; /* write unlock cycle 1 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x2AA ) = 0x0055; /* write unlock cycle 2 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x0080; /* write setup command */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x00AA; /* write additional unlock cycle 1 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x2AA ) = 0x0055; /* write additional unlock cycle 2 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x0010; /* write chip erase command */

	while(1) {
		rdata1 = (*(volatile unsigned short *)FLASH_CONTOLLER_BASE + 0);
		rdata2 = (*(volatile unsigned short *)FLASH_CONTOLLER_BASE + 0);
		if( (rdata1 == 0xFFFF) && (rdata2 == 0xFFFF)) {
			break;
		}
		timeout++;
	}
	if( timeout == FLASH_TIMEOUT ) {
		return -1;
	}
	return 0;
}

int flash_sector_erase(unsigned int sector )
{
	unsigned short rdata1;
	int timeout = 0;

	/* Note: Cannot be suspended */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x00AA; /* write unlock cycle 1 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x2AA ) = 0x0055; /* write unlock cycle 2 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x0080; /* write setup command */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x00AA; /* write additional unlock cycle 1 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x2AA ) = 0x0055; /* write additional unlock cycle 2 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + (sector*FLASH_SECTOR_SIZE >> 1) ) = 0x0030; /* write sector erase command */

	while(1) {
		rdata1 = *((volatile unsigned short *)(FLASH_CONTOLLER_BASE + FLASH_SECTOR_SIZE*sector) );
		if( rdata1 == 0xFFFF ) {
			break;
		}
		timeout++;
	}
	if( timeout == FLASH_TIMEOUT ) {
		return -1;
	}
	return 0;
}

//write 2 bytes data to flash using offset addr
int flash_write( unsigned int offset_addr, unsigned short data )
{
	unsigned short rdata1;
	int timeout = 0;

	offset_addr >>= 1;
	/* Program Command */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x00AA; /* write unlock cycle 1 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x2AA ) = 0x0055; /* write unlock cycle 2 */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + 0x555 ) = 0x00A0; /* write program setup command */
	*( (volatile unsigned short *)FLASH_CONTOLLER_BASE + offset_addr ) = data;    /* write data to be programmed */

	while(1) {
		rdata1 = *( (volatile unsigned short *)FLASH_CONTOLLER_BASE + (unsigned int)offset_addr );
		if( rdata1 == data ) {
			break;
		}
		timeout++;
	}
	if( timeout == FLASH_TIMEOUT ) {
		return -1;
	}

	return 0;
}

unsigned short flash_read( unsigned int addr )
{
	unsigned short *p;

	p = (unsigned short *)(FLASH_CONTOLLER_BASE+addr);
	return *p;
}

//Flash write and read back check
int flash_check(void)
{
	unsigned int flash_offset, sector;
	unsigned short value;

//Read and Write check all data in 1st sector of flash.
//printf( "FLASH check 1st sector all word check\n" );
	flash_chip_erase();
//printf( "chip erase complete\n" );

	sector = 0;
//	flash_sector_erase( sector );
	for( flash_offset = 0, value = 0; flash_offset < FLASH_SECTOR_SIZE; flash_offset+=2, value++) {
		flash_write( sector*FLASH_SECTOR_SIZE + flash_offset, value );
//		if( (flash_offset & 0xfff) == 0) {
//			printf("flash_offset=%x\n", flash_offset );
//		}
	}
	for( flash_offset = 0, value = 0; flash_offset < FLASH_SECTOR_SIZE; flash_offset+=2, value++) {
		if( flash_read( sector*FLASH_SECTOR_SIZE + flash_offset ) != value ) {
			return -1;								//Fail
		}
	}

//All sector 1 word write and eread check
//printf( "FLASH check all sector one word write check\n" );
//
	for( sector = 1; sector < NUMBER_OF_SECTORS; sector++ ) {
//		flash_sector_erase( sector );
		flash_write( sector*FLASH_SECTOR_SIZE + TEST_OFFSET, 0xa5a5 );
//
//printf("FLASH check sector=%d\n", sector );
		if( flash_read( sector*FLASH_SECTOR_SIZE + TEST_OFFSET ) != 0xa5a5 ) {
			return -1;
		}
	}
	return 0;
}

void check_7seg_leds(int seg_led_number,  // 0 7seg #0, 1 7seg #1
		             unsigned char disp_num )      // 0 to F
{
	unsigned char *base_ptr;
	unsigned char data_7seg;


	if( seg_led_number == 1 ) {			//7SEG No1
		base_ptr = (unsigned char *)SEVEN_SEG_1_BASE;
	}
	else {								//7SEG No0
		base_ptr = (unsigned char *)SEVEN_SEG_0_BASE;
	}
	switch ( disp_num ) {
		case 0x0:
			data_7seg = 0xc0;
			break;
		case 0x1:
			data_7seg = 0xf9;
			break;
		case 0x2:
			data_7seg = 0xa4;
			break;
		case 0x3:
			data_7seg = 0xb0;
			break;
		case 0x4:
			data_7seg = 0x99;
			break;
		case 0x5:
			data_7seg = 0x92;
			break;
		case 0x6:
			data_7seg = 0x82;
			break;
		case 0x7:
			data_7seg = 0xf8;
			break;
		case 0x8:
			data_7seg = 0x80;
			break;
		case 0x9:
			data_7seg = 0x98;
			break;
		case 0xa:
			data_7seg = 0x88;
			break;
		case 0xb:
			data_7seg = 0x83;
			break;
		case 0xc:
			data_7seg = 0xa7;
			break;
		case 0xd:
			data_7seg = 0xa1;
			break;
		case 0xe:
			data_7seg = 0x86;
			break;
		case 0xf:
			data_7seg = 0x8e;
			break;
		case 0x10:
			data_7seg = 0x7f;
			break;
		default:
			data_7seg = 0xff;

	}
	*base_ptr = data_7seg; // write initial value to pio
//	IOWR_ALTERA_AVALON_PIO_DATA(base_ptr,data_7seg); // write initial value to pio
}


void check_7_seg_led(void)
{
	//endress loop for 0-f for each 7Seg LEDs
	int i;

	while(1) {
		for( i = 0; i < 0x10; i++) {
		    check_7seg_leds(0,i);
		    check_7seg_leds(1,i);
		    usleep(1000000);
		}
	}
}


int main()
{
//	check_7seg_leds(1, 0x8);
//	check_7seg_leds(0, 0x8);

#if 1
	//DDR Check
	if( ddr_check() == 0 ) {
		*p_led_select0 = 0x1;		//LED0 ON PASS
	}
	else {
		*p_led_select0 = 0x2;		//LED0 BLINK FAIL
	}
#endif

	//SRAM _Check
#if 1
	if( sram_check() == 0 ) {
		*p_led_select1 = 0x1;		//LED1 ON PASS
	}
	else {
		*p_led_select1 = 0x2;		//LED1 BLINK FAIL
	}
#endif

	//FLASH check
#if 1
    if ( flash_check() == 0 ) {
		*p_led_select2 = 0x1;		//LED1 ON PASS
	}
	else {
		*p_led_select2 = 0x2;		//LED1 BLINK FAIL
	}
#endif
 //   flash_check();
    check_7_seg_led();

    return 0;
}
